set.seed(1)

rescale_design = function(design, lower, upper) {
  n = ncol(design)
  for(i in 1:n) {
    design[, i] = design[, i] * (upper[i] - lower[i]) + lower[i]
  }
  design = as.data.frame(design)
  colnames(design) = paste0("x", 1:n)
  return(design)
}

generate_lhd = function(n, lower, upper) {
  # build a lhd:
  design = lhs::improvedLHS(n = n, k = length(lower))
  design = rescale_design(design = design, lower = lower, upper = upper)
  return(design)
}

################################################################################

# objective function 1: 
f1 = function(x) {
  return(10 - x * cos(x))
}

# data1:
data1 = data.frame(x = seq(from = -10, to = 10, by = 2),
                   value = f1(seq(from = -10, to = 10, by = 2)))

# grid1:
grid1 = expand.grid(x = seq(from = -10, to = 10, by = 0.1))

# true1:
true1 = f1(grid1)$x

################################################################################

# objective function 2:
f2 = function(x1, x2, p = 5, q = 3) {
  return(p * pmin(x1, x2) - q * x1)
} 

# data2:
data2 = cbind(x1 = 0:100, x2 = rnorm(n = 101, mean = 50, sd = 7))
data2 = data.frame(data2, value = f2(data2[, 1], data2[, 2]))

# grid2:
grid2 = expand.grid(x1 = seq(from = 0, to = 100, by = 1),
                    x2 = seq(from = 30, to = 70, by = 1))

# true2:
true2 = f2(grid2$x1, grid2$x2)

################################################################################

# objective function 3: 
f3 = function(x1, x2, x3, x4, p = 5, q = 3, r = 4, s = 2) {
  return(p * pmin(x1, x3) - q * x1 + r * pmin(x2, x4) - s * x2)
}

# data3:
data3 = generate_lhd(n = 250, lower = c(0, 20), upper = c(100, 120))
data3 = data.frame(data3, mvtnorm::rmvnorm(n = 250, mean = c(50, 70), 
                                           sigma = matrix(c(9, -6, -6, 9), nrow = 2)))
data3 = cbind(data3, value = f3(x1 = data3[, 1], x2 = data3[, 2],
                                x3 = data3[, 3], x4 = data3[, 4]))
colnames(data3)[3:4] = c("x3", "x4")

# grid3:
grid3 = generate_lhd(n = 1e3, lower = c(0, 20, 40, 60), upper = c(100, 120, 60, 80))

# true3: 
true3 = f3(x1 = grid3[, 1], x2 = grid3[, 2], x3 = grid3[, 3], x4 = grid3[, 4])

# save(data1, data2, data3, grid1, grid2, grid3, true1, true2, true3, file = "po21_blatt08_data.RData")
